﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PuzzlePlayer_Namespace
{
    internal class PuzzleForm : Form
    {
        public readonly string puzzleType;
        private readonly Button solvebutton;
        private readonly Button hintbutton;
        private readonly Button generatebutton;
        private readonly Label titlebox;
        private readonly Label informationbox;
        private Rectangle boardspace;
        private readonly BufferedGraphics bufferedGraphics;
        private Board board;
        public Board Board //updating the Board member will immediately call board.Draw method so that the board is updated visually

        {
            get { return board; }
            set
            {
                board = value;
                this.Invalidate();
            }
        }

        private readonly Button UPDATEBUTTON;


        public PuzzleForm(Board b, Size s = default) //takes Board and Size parameter and sets up the PuzzleForm window.
        {
            puzzleType = b.GetType().Name;
            if (s == default) s = new Size(700, 420);
            this.Size = this.MinimumSize = s;
            this.WindowState = FormWindowState.Maximized;
            this.BackColor = Color.DarkGreen;
            this.Text = "PuzzlePlayer";
            bufferedGraphics = BufferedGraphicsManager.Current.Allocate(this.CreateGraphics(), this.DisplayRectangle);
            bufferedGraphics.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            generatebutton = new Button();
            hintbutton = new Button();
            solvebutton = new Button();
            UPDATEBUTTON = new Button();
            titlebox = new Label();
            informationbox = new Label();
            boardspace = new Rectangle(220, 30, 400, 400);

            this.Paint += (object o, PaintEventArgs pea) =>
            {
                board.Draw(bufferedGraphics.Graphics, boardspace);
                //bufferedGraphics.Graphics.FillRectangle(Brushes.LightCoral, 220, 30, this.Width - 350, this.Height - 100);
                //bufferedGraphics.Graphics.FillRectangle(Brushes.DarkRed, boardspace);
                bufferedGraphics.Render();
            };
            this.Resize += (object o, EventArgs ea) => UpdateUI();
            this.Move += (object o, EventArgs ea) => this.Focus();
            this.KeyPress += (object o, KeyPressEventArgs kea) => Input(kea.KeyChar);
            this.MouseClick += (object o, MouseEventArgs mea) =>
            {
                if (mea.Button == MouseButtons.Left)
                {
                    Input('[');
                    return;
                }
                if (mea.Button == MouseButtons.Right)
                {
                    Input(']');
                    return;
                }
            };

            Board = b;
            CreateUI();
        }
        private void CreateUI() //sets up ui elements
        {
            void CreateButton(Button b)
            {
                Controls.Add(b);
                b.Size = new Size(80, 50);
                b.FlatAppearance.BorderColor = b.BackColor = Color.Gainsboro;
                b.Text = "DEFAULT0";
                b.Font = new Font("Verdana", 12, FontStyle.Bold | FontStyle.Italic);
                b.FlatStyle = FlatStyle.Flat;
                b.GotFocus += (object o, EventArgs ea) => this.Focus();
            }
            CreateButton(generatebutton);
            generatebutton.Text = "New Game";
            generatebutton.MouseClick += (object sender, MouseEventArgs mea) => 
            {
                Board.Generate();
                Board = Board;
            };
            CreateButton(hintbutton);
            hintbutton.Text = "Hint";
            hintbutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                MessageBox.Show("Hint: geef op");
                Board = Board;
            };
            CreateButton(solvebutton);
            solvebutton.Text = "Solve";
            solvebutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                Board.Solve(false);
                Board = Board;
            };

            CreateButton(UPDATEBUTTON);
            UPDATEBUTTON.Text = "Update";
            UPDATEBUTTON.Font = new Font("Verdana", 11, FontStyle.Bold);
            UPDATEBUTTON.FlatAppearance.BorderColor = UPDATEBUTTON.BackColor = Color.Pink;
            UPDATEBUTTON.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                Board = Board;
            };

            Controls.Add(titlebox);
            titlebox.Location = new Point(20, 15);
            titlebox.Size = new Size(180, 50);
            titlebox.BackColor = this.BackColor;
            titlebox.ForeColor = Color.White;
            titlebox.Text = "Binary";
            titlebox.Font = new Font("Verdana", 24, FontStyle.Bold);

            Controls.Add(informationbox);
            informationbox.Location = new Point(20, 65);
            informationbox.Size = new Size(180, 300);
            informationbox.BackColor = this.BackColor;
            informationbox.ForeColor = Color.White;
            informationbox.Text = board.description;
            informationbox.Font = new Font("Verdana", 10);
            UpdateUI();
        }
        private void UpdateUI() //resizes the boardspace rectangle and updates the rest of the ui around the new boardspace size
        {
            bufferedGraphics.Graphics.Clear(this.BackColor);
            boardspace.Size = FitBoard(new Size(Board.boardState.GetLength(0),Board.boardState.GetLength(1)),new Size(this.Width - 350, this.Height - 100),8);
            generatebutton.Location = new Point(boardspace.Right + 30, 30);
            hintbutton.Location = new Point(boardspace.Right + 30, 110);
            solvebutton.Location = new Point(boardspace.Right + 30, 190);
            UPDATEBUTTON.Location = new Point(boardspace.Right + 30, 270);
            this.Invalidate();
        }
        public static Size FitBoard(Size gamesize, Size maxboardsize, int drawfactor) //returns the largest rectangle smaller than MaxBoardSize that fits the given GameSize well
        {
            int TileLength = Math.Min(maxboardsize.Width / gamesize.Width, maxboardsize.Height / gamesize.Height) / drawfactor * drawfactor;
            return new Size(gamesize.Width*TileLength, gamesize.Height*TileLength);
            //after each of the divisions in this method, the value should be rounded down, 
            //as the end value cannot exceed the size given by maxboardsize.
            //however, C# rounds divisions of Int values down by default, so this does not have to be explicitly done.
        }
        private Point GetTile(Size gamesize, Rectangle r, Point p) //returns board coordinate at mouse position P of a Gamesize board drawn in R
        {
            Point res = this.PointToClient(p) - (Size)r.Location;
            res.X /= (r.Width / gamesize.Width);
            res.Y /= (r.Height / gamesize.Height);
            return res;
        }
        public void Input(char c) //checks if a command binded to the keyboard key and runs it, affects tile that is hovered on if applicable
        {
            if (!"nhs[]1234567890".Contains(c)) return;
            switch (c)
            {
                case 'n':
                    return;
                case 'h':
                    MessageBox.Show("Hint: geef op");
                    return;
                case 's':
                    return;

            }
            Point tile = GetTile(new Size(Board.boardState.GetLength(0), Board.boardState.GetLength(1)), boardspace, Control.MousePosition);
            if (!(tile.X >= 0 && tile.X < Board.boardState.GetLength(0) && tile.Y >= 0 && tile.Y < Board.boardState.GetLength(1))) return;
            if (c == '[' || c == ']') // '[' = 91, ']' = 93
            {
                //Board.TileClick(tile, (c - 91) / 2);
                return;
            }
            if (char.GetNumericValue(c) != -1)
            {
                Board.TileInput(tile, (int)char.GetNumericValue(c));
                return;
            }
            MessageBox.Show("uhoh");
        }
    }
}
